# $Id: ejabberd.tcl,v 1.1 2003/11/06 20:51:40 aleksey Exp $

package require msgcat

set sysenc [encoding system]
encoding system utf-8
::msgcat::mcload [file join [file dirname [info script]] msgs]
encoding system $sysenc
unset sysenc

namespace eval ejabberd {
    custom::defvar ejabberd_server_list {} \
	[::msgcat::mc "List of ejabberd servers."] \
	-group Hidden
}

set ::NS_ECONFIGURE "http://ejabberd.jabberstudio.org/protocol/configure"

proc ejabberd::setup_menu {} {
    catch { 
        set m [.mainframe getmenu admin]

        $m add command -label [::msgcat::mc "Administrate ejabberd"] \
	    -command [namespace current]::ask_server_dialog
    }
}
hook::add finload_hook [namespace current]::ejabberd::setup_menu


proc ejabberd::ask_server_dialog {} {
    global ejabberd_server
    variable ejabberd_server_list

    set gw .ejabberdserver
    catch { destroy $gw }

    Dialog $gw -title [::msgcat::mc "ejabberd server"] -separator 1 -anchor e \
	    -default 0 -cancel 1

    set gf [$gw getframe]
    grid columnconfigure $gf 1 -weight 1

    if {[llength $ejabberd_server_list]} {
	set ejabberd_server [lindex $ejabberd_server_list 0]
    }

    label $gf.ljid -text [::msgcat::mc "JID:"]
    ecursor_entry [ComboBox $gf.jid -textvariable ejabberd_server \
		       -values $ejabberd_server_list -width 35].e
    
    grid $gf.ljid -row 0 -column 0 -sticky e
    grid $gf.jid  -row 0 -column 1 -sticky ew
		    
    $gw add -text [::msgcat::mc "Administrate"] -command "[namespace current]::administrate $gw"
    $gw add -text [::msgcat::mc "Cancel"] -command "destroy $gw"

    $gw draw $gf.jid
}

proc ejabberd::administrate {gw} {
    global ejabberd_server
    variable ejabberd_server_list

    destroy $gw
    set ejabberd_server_list \
	[update_combo_list $ejabberd_server_list $ejabberd_server 10]
    set custom::saved([namespace current]::ejabberd_server_list) \
	$ejabberd_server_list
    custom::store
    open_win $ejabberd_server
}


proc ejabberd::open_win {jid} {

    set w [win_id ejabberd $jid]
    if {[winfo exists $w]} {
	return
    }

    set title [format [::msgcat::mc "%s administration"] $jid]
    add_win $w -title $title -tabtitle $jid \
	-class Ejabberd \
	-raisecmd [list tab_set_updated $w]


    set nb [NoteBook $w.nb]
    pack $nb -fill both -expand yes


    foreach {page title} {
	main "Main"
	nodes "Nodes"
	reg "Registration"
	access "Access"
	last "Last Activity"
    } {
	set f [$nb insert end $page -text [::msgcat::mc $title]]

	fill_page_$page $f $jid
    }
    $nb raise main
}

proc ejabberd::add_grid_record {jid info name desc row} {
    label $info.l$name -text [::msgcat::mc $desc]
    label $info.v$name -textvariable [namespace current]::data($jid,$name)
    grid $info.l$name -row $row -column 0 -sticky e
    grid $info.v$name -row $row -column 1 -sticky w
}

proc ejabberd::add_grid_edit {jid info name desc row} {
    label $info.l$name -text [::msgcat::mc $desc]
    entry $info.v$name -textvariable [namespace current]::data($jid,$name)
    grid $info.l$name -row $row -column 0 -sticky e
    grid $info.v$name -row $row -column 1 -sticky we
}

proc ejabberd::add_grid_text {jid info name desc row} {
    global font
    label $info.l$name -text [::msgcat::mc $desc]
    set sw [ScrolledWindow $info.s$name -scrollbar vertical]
    text $info.v$name -height 6 -wrap word -font $font
    $sw setwidget $info.v$name
    grid $info.l$name -row $row -column 0 -sticky e
    grid $info.s$name -row $row -column 1 -sticky we
}

proc ejabberd::fill_page_main {f jid} {
    variable data

    set info [frame $f.info]
    pack $info -side top -anchor w -fill both

    grid columnconfigure $info 1 -weight 2

    add_grid_record $jid $info total_users "Registered users:" 0
    add_grid_record $jid $info online_users "Online users:" 1
    add_grid_record $jid $info running_nodes "Running nodes:" 2
    add_grid_record $jid $info stopped_nodes "Stopped nodes:" 3
    add_grid_record $jid $info outgoing_s2s "Outgoing S2S:" 4

    set reload \
	[button $f.reload -text [::msgcat::mc "Reload"] \
	     -command [list [namespace current]::reload_page_main $f $jid]]
    pack $reload -side bottom -anchor e
    reload_page_main $f $jid
}

proc ejabberd::reload_page_main {f jid} {
    jlib::send_iq get \
	[jlib::wrapper:createtag info \
	     -vars [list xmlns $::NS_ECONFIGURE]] \
	-to $jid \
	-command [list [namespace current]::parse_main_info $f $jid]
}

proc ejabberd::parse_main_info {f jid res child} {
    variable data

    if {![cequal $res OK]} {
	return
    }

    jlib::wrapper:splitxml $child tag vars isempty chdata children

    set data($jid,total_users) [jlib::wrapper:getattr $vars registered-users]
    set data($jid,online_users) [jlib::wrapper:getattr $vars online-users]
    set data($jid,running_nodes) [jlib::wrapper:getattr $vars running-nodes]
    set data($jid,stopped_nodes) [jlib::wrapper:getattr $vars stopped-nodes]
    set data($jid,outgoing_s2s) [jlib::wrapper:getattr $vars outgoing-s2s-servers]
}


proc ejabberd::fill_page_nodes {f jid} {
}

proc ejabberd::fill_page_reg {f jid} {
    variable data

    set info [frame $f.info]
    pack $info -side top -anchor w -fill both

    grid columnconfigure $info 1 -weight 2

    add_grid_edit $jid $info welcome_subj "Welcome message subject:" 0
    add_grid_text $jid $info welcome_body "Welcome message body:" 1
    add_grid_text $jid $info reg_watchers "Registration watchers:" 2


    #set set_b [button $f.set -text [::msgcat::mc "Set"]]
    #pack $set_b -side right -anchor se
    set reload \
	[button $f.reload -text [::msgcat::mc "Reload"] \
	     -command [list [namespace current]::reload_page_reg $f $jid]]
    pack $reload -side right -anchor se
    reload_page_reg $f $jid
}

proc ejabberd::reload_page_reg {f jid} {
    jlib::send_iq get \
	[jlib::wrapper:createtag welcome-message \
	     -vars [list xmlns $::NS_ECONFIGURE]] \
	-to $jid \
	-command [list [namespace current]::parse_welcome_message $f $jid]
    jlib::send_iq get \
	[jlib::wrapper:createtag registration-watchers \
	     -vars [list xmlns $::NS_ECONFIGURE]] \
	-to $jid \
	-command [list [namespace current]::parse_registration_watchers $f $jid]
}

proc ejabberd::parse_welcome_message {f jid res child} {
    variable data

    set wsubj $f.info.vwelcome_subj
    set wbody $f.info.vwelcome_body

    if {![winfo exists $wsubj]} {
	return
    }

    if {![cequal $res OK]} {
	return
    }

    jlib::wrapper:splitxml $child tag vars isempty chdata children

    set subj ""
    set body ""

    foreach ch $children {
	jlib::wrapper:splitxml $ch tag1 vars1 isempty1 chdata1 children1

	switch -- $tag1 {
	    subject {set subj $chdata1}
	    body {set body $chdata1}
	}
    }

    set data($jid,welcome_subj) $subj
    set data($jid,welcome_body) $body
    $wbody delete 0.0 end
    $wbody insert 0.0 $body
}


proc ejabberd::parse_registration_watchers {f jid res child} {
    variable data

    set wwatchers $f.info.vreg_watchers

    if {![winfo exists $wwatchers]} {
	return
    }

    if {![cequal $res OK]} {
	return
    }

    jlib::wrapper:splitxml $child tag vars isempty chdata children

    set jids {}

    foreach ch $children {
	jlib::wrapper:splitxml $ch tag1 vars1 isempty1 chdata1 children1

	switch -- $tag1 {
	    jid {lappend jids $chdata1}
	}
    }

    set data($jid,reg_watchers) $jids
    $wwatchers delete 0.0 end
    $wwatchers insert 0.0 [join $jids \n]
}



proc ejabberd::fill_page_access {f jid} {
    variable data

    set info [frame $f.info]
    pack $info -side top -anchor w -fill both

    grid columnconfigure $info 1 -weight 2

    add_grid_text $jid $info acls "ACLs:" 0
    add_grid_text $jid $info access_rules "Access rules:" 1


    #set set_b [button $f.set -text [::msgcat::mc "Set"]]
    #pack $set_b -side right -anchor se
    set reload \
	[button $f.reload -text [::msgcat::mc "Reload"] \
	     -command [list [namespace current]::reload_page_access $f $jid]]
    pack $reload -side right -anchor se

    reload_page_access $f $jid
}

proc ejabberd::reload_page_access {f jid} {
    jlib::send_iq get \
	[jlib::wrapper:createtag acls \
	     -vars [list xmlns $::NS_ECONFIGURE]] \
	-to $jid \
	-command [list [namespace current]::parse_access acls $f $jid]
    jlib::send_iq get \
	[jlib::wrapper:createtag access \
	     -vars [list xmlns $::NS_ECONFIGURE]] \
	-to $jid \
	-command [list [namespace current]::parse_access access_rules $f $jid]
}

proc ejabberd::parse_access {var f jid res child} {
    variable data

    set w $f.info.v$var

    if {![winfo exists $w]} {
	return
    }

    if {![cequal $res OK]} {
	return
    }

    jlib::wrapper:splitxml $child tag vars isempty chdata children

    set data($jid,$var) $chdata
    $w delete 0.0 end
    $w insert 0.0 $chdata
}


proc ejabberd::fill_page_last {f jid} {
    variable data

    set info [frame $f.info]
    pack $info -side top -anchor w -fill both -expand yes

    #grid columnconfigure $info 1 -weight 2

    #add_grid_text $jid $info acls "ACLs:" 0
    #add_grid_text $jid $info access_rules "Access rules:" 1

    label $info.lplot \
	-text [::msgcat::mc \
		   "Number of users that used this service N days ago:"]
    pack $info.lplot -side top -anchor w

    set sw [ScrolledWindow $info.sw]
    pack $sw -side top -fill both -expand yes
    set plot [canvas $info.plot -background white]
    $sw setwidget $plot

    set data($jid,last) {}
    set data($jid,last_int) 0

    set integral \
	[checkbutton $f.integral -text [::msgcat::mc "Integral"] \
	     -variable [namespace current]::data($jid,last_int) \
	     -command [list [namespace current]::redraw_last $f $jid]]
    pack $integral -side left -anchor se

    set reload \
	[button $f.reload -text [::msgcat::mc "Reload"] \
	     -command [list [namespace current]::reload_page_last $f $jid]]
    pack $reload -side right -anchor se
}

proc ejabberd::reload_page_last {f jid} {
    jlib::send_iq get \
	[jlib::wrapper:createtag last \
	     -vars [list xmlns $::NS_ECONFIGURE]] \
	-to $jid \
	-command [list [namespace current]::parse_last $f $jid]
}

proc ejabberd::parse_last {f jid res child} {
    variable data

    set plot $f.info.plot

    if {![winfo exists $plot]} {
	return
    }

    if {![cequal $res OK]} {
	return
    }

    jlib::wrapper:splitxml $child tag vars isempty chdata children

    set data($jid,last) $chdata

    redraw_last $f $jid
}

proc ejabberd::redraw_last {f jid} {
    variable data

    set plot $f.info.plot

    set maxdays 0

    foreach t $data($jid,last) {
	set days [expr {$t / (60*60*24)}]
	if {$days > $maxdays} {set maxdays $days}

	if {![info exists last($days)]} {
	    set last($days) 0
	}
	incr last($days)
    }

    set xscale 20
    set yscale 200

    $plot delete all

    set val 0

    for {set i 0} {$i <= $maxdays} {incr i} {
	if {[info exists last($i)]} {
	    set v $last($i)
	} else {
	    set v 0
	}

	if {$data($jid,last_int)} {
	    incr val $v
	} else {
	    set val $v
	}

	set x1 [expr {$xscale * $i}]
	set x2 [expr {$xscale * ($i+1)}]
	set x [expr {($x1 + $x2)/2}]
	set y [expr {-$yscale * $val}]
	
	$plot create rectangle $x1 0 $x2 $y -fill red

	$plot create text $x 0 -text $i -anchor n
	$plot create text $x $y -text $val -anchor s
    }

    set bbox [$plot bbox all]
    
    set y1 [lindex $bbox 1]
    set y2 [lindex $bbox 3]
    set height [winfo height $plot]

    $plot scale all 0 0 1 [expr {0.9 * $height / (0.0 + $y2 - $y1)}]
    $plot configure -scrollregion [$plot bbox all]
}
